from __future__ import annotations

import random
import string
from typing import Iterator


class Passworder:
    """Passworder."""

    DEFAULT_SEPARATOR = "\n"

    def __init__(
        self,
        separator: str = DEFAULT_SEPARATOR,
        with_upper: bool = False,
        with_digits: bool = False,
        with_specials: bool = False,
    ) -> None:
        self._alphabet: tuple[str, ...] = self._get_alphabet(
            with_digits=with_digits,
            with_specials=with_specials,
            with_upper=with_upper,
        )

        self._random = random.SystemRandom()

        self._separator = separator

        self._with_upper = with_upper
        self._with_digits = with_digits
        self._with_specials = with_specials

    @staticmethod
    def _get_alphabet(
        with_upper: bool = False,
        with_digits: bool = False,
        with_specials: bool = False,
    ) -> tuple[str, ...]:
        alphabet = list(string.ascii_lowercase)

        if with_upper:
            alphabet.extend(list(string.ascii_uppercase))

        if with_digits:
            alphabet.extend(list(string.digits))

        if with_specials:
            alphabet.extend(list(string.punctuation))

        return tuple(alphabet)

    def _get_sequences(self, count: int, length: int) -> Iterator[str]:
        """Generator sequences.

        Args:
            count (int): sequences count
            length (int): symbols length per sequence

        Yields:
            str: sequence
        """
        for _ in range(count):
            result = None

            while not result:
                result = "".join(self._random.choice(self._alphabet) for _ in range(length))

            yield result

    def get(self, count: int, length: int) -> str:
        """Get separated sequences.

        Args:
            count (int): sequences count
            length (int): sequence length
            separator (str, optional): Defaults to '\n'.

        Returns:
            str: Sequences
        """
        pass_sequences: list[str] = []

        for seq in self._get_sequences(count, length):
            pass_sequences.append(seq)

        return f"{self._separator}".join(pass_sequences)



import argparse

from fastpass.__about__ import __project__, __version__
from fastpass.core.passworder import Passworder


class App:
    """App entrypoint."""

    def __init__(self) -> None:
        self._parser = argparse.ArgumentParser(prog=__project__)

    def start(self) -> None:
        """Start parse args and set to pass."""
        self._set_commands()

        command = self._parser.parse_args()

        if command.version:
            self.version()

        else:
            self._root(
                count=command.count,
                length=command.length,
                separator=command.separator,
            )

    def _set_commands(self) -> None:
        """Set command args."""

        self._parser.add_argument("--version", "-V", action="store_true", help="Print fastpass version.")

        self._parser.add_argument("--count", "-c", type=int, default=1, help="Number of passwords for generate.")
        self._parser.add_argument(
            "--length",
            "-l",
            type=int,
            default=5,
            help="Length of generated passwords.",
        )

        self._parser.add_argument(
            "--separator",
            "-S",
            type=str,
            default="\n",
            help="Separator between passwords.",
        )

    def version(self) -> None:
        """Show version."""
        print(__version__)

    def _root(  # noqa: CFQ002
        self,
        count: int = 1,
        length: int = 5,
        separator: str = "-",
        with_digits: bool = True,
        with_upper: bool = True,
        with_specials: bool = False,
    ) -> None:
        """Generate passwords."""

        generator = Passworder(
            separator=separator,
            with_upper=with_upper,
            with_digits=with_digits,
            with_specials=with_specials,
        )

        print(generator.get(count=count, length=length))
